##
# get_exten
#
# get the location of an extension
# based on the internal or extenral.conf
# Extracted from old getexten.agi
#
# Felix Nilam - 08/02/2009
##
sub get_exten
{
	# grab the extension
	my $exten = shift;        
	my $server = shift;
	
	return 0 if ($exten !~ /^\d+$/);
	
	my $prefix;
	$prefix = "/data/$server" if ($server =~ /^\d+$/);
	$prefix .= "/etc/asterisk/fonality";

	# check if extension resides on local server
	foreach my $conf ('external', 'internal') {
		my $return = check_conf("$prefix/$conf.conf", $exten, 0);
		return $return if(defined($return));
	}

	# check if extension resides on a linked server
	foreach my $conf ('link-internal', 'link-external', 'link') {
		my $return = check_conf("$prefix/$conf.conf", $exten, 1);
		return $return if(defined($return));
	}
	
	# if it reaches this point, extension is not found
	return 0;
}

# Felix Nilam - 08/25/2009
# check_conf
# check the conf file and return the location and linked server
#
# Arguments: conf_file,
#			 extension,
#            link   0 or 1 flag if it is link, link-external or link-internal
# Returns: hasref(location, linked_server) or undef if not found
sub check_conf
{
	my $conf_file = shift;
	my $exten     = shift;
	my $link      = shift;

	open(CONF, "<$conf_file") || die "Can't read $conf_file: $!";
	
	# split the extension into individual digits
	my @digits = split(//, $exten);
	# generate the pattern matching
	my $pattern = '';
	foreach my $dig (@digits)
	{
		$pattern .= '['.$dig.'X]';
	}
	
	while (<CONF>)
	{
		if($link == 1)
		{
			if (/^exten => _$pattern,1,(Macro\([^,]+,[^,]+,[^,]+,[^),]+\))/)
			{
				close CONF;

				# split the macro
				my $macr = $1;
				$macr =~ s/^Macro\(//g;
				$macr =~ s/\)$//g;
				my @macro = split(/,/, $macr);
				
				return $macro[2];
			}
		}
		else
		{
			if (/^exten => $exten,1,Macro\([^,]+,[^,]+,[^,]+,([^),]+)/) 
			{
				close CONF;
				return $1;
			}
		}
	}
	close CONF;
	return undef;
}

sub prepend_num
{
	require '/var/lib/asterisk/agi-bin/rights.pl';
	
	my $number = shift;
	my $server = shift;
	
	dbg("number/server passed: $number/$server");
	
	my $prefix;
	$prefix = "/data/$server" if($server =~ /^\d+/);
	$prefix .= "/etc/asterisk/fonality";
	
	my @handles;
	my $out;
	
	foreach ('trunkld', 'trunklocal', 'trunktollfree', 'trunkint') {
		open(my $fh, "<$prefix/$_.conf") || return "Can't read $conf_file: $!";
		push(@handles, $fh);
	}

	# loop through the trunk conf files
	foreach my $ioh (@handles) {
		my $cur;
		while(<$ioh>) {
			my $dp;
			$dp = $1 if(/^exten => ([^,]+)/);
			
			# we are looking for unique dialplan patterns,
			# so skip if pattern not found or we've seen it already
			next unless($dp);
			next if($cur =~ /^$dp$/);
			$cur = $dp;
			
			dbg("looking at: $dp");
			
			# disregard leading _
			$dp =~ s/^_//;

			# split pattern by first digit (prepending number) and the rest
			my $pre_num;
			($pre_num, $dp) = ($1, $2) if($dp =~ /^(\d)(.*)/);

			# convert symbols into their respective digit classes
			$dp =~ s/n/\[2-9\]/ig;
			$dp =~ s/z/\[1-9\]/ig;
			$dp =~ s/x/\\d/ig;
			$dp =~ s/\./\\d+/g;
			$dp =~ s/!/\\d*/g;
		
			dbg("converted to: $dp");	
			
			if($number =~ /^$dp$/) {
				$out = $pre_num . $number;
				last;
			} elsif($number =~ /$pre_num$dp/) {
				$out = $number;
				last;
			}
		}

		last if $out;
	}
	
	# close the handles
	foreach (@handles) {
		close $_;
	}
	
	dbg("returning $out || $number");
	return $out || $number;
}

1;
