#!/usr/bin/perl
#
# used for recording all calls from/to certain extensions
#
# v0.18, 08/02/06
#
# -samyk, 06/18/06

# DON'T USE THE MANAGER -- ASTERISK BUG CAUSES ASTERISK TO LAG WHEN USING
# MANAGER DIRECTLY FROM A FASTAGI, FROM NORMAL AGI ITS OK
my $manager = 0;	# use Asterisk Manager API vs normal command execution
my $getName = 1;	# should we get the name of the extension and append it to filename?
my $inclusion = "/etc/asterisk/record_all.conf";

my $port = 6969;

package server;

use strict;
use vars qw(@ISA);
use Net::Server::PreFork;
use IO::Socket;

@ISA = qw(Net::Server::PreFork);

# run the server
server->run();
exit;


# set up some server parameters
sub configure_hook
{
	my $self = shift;

	$self->{server}->{port}		= ["*:$port"];		# port and addr to bind
	$self->{server}->{chdir}	= '/';			# chdir to root
	$self->{server}->{user}		= 'nobody';		# user to run as
	$self->{server}->{group}	= 'nobody';		# group to run as
	$self->{server}->{setsid}	= 1;			# daemonize

	open(STDIN,  '</dev/null') || die "Can't close STDIN [$!]";
	open(STDOUT, '>/dev/null') || die "Can't close STDOUT [$!]";
#	open(STDERR, '>&STDOUT')   || die "Can't close STDERR [$!]";
}

# do this when the connection is made
sub post_accept_hook
{
	my $self = shift;
}


# process the request
sub process_request
{
	my $self = shift;
	my %data;

	# read in agi params
	while (<STDIN>)
	{
open F,">>/tmp/deb";print F "\t$_";close(F);
		# standard agi format of data passed
		if (/^(\S+): (.*)$/)
		{
			$data{$1} = $2;
		}
		else
		{
			last;
		}
	}

	if ($manager)
	{
		$self->mgr_login(1) || return;
	}

	# do we have a channel variable (queue call)?
	my $channel = $self->get_var("CHAN");
	if (defined($channel))
	{
		# we have a queue call, everything should already be handled
		$self->process_queue(\%data, $channel);
		return;
	}

	my $cidnum  = ($data{agi_callerid} =~ /<(\d+)>/ ? $1 : $data{agi_callerid});
	my $exten   = $self->get_var("EXTEN") || $data{agi_extension};
	my $channel = $data{agi_channel};
	my $number  = $self->get_var("NUMBER");

	# we can make this faster by doing a stat comparison of time stamp if we want
	open(INCL, "<$inclusion") || error("Can't read inclusion file $inclusion: $!");

	my $fmt = "wav";
	my $mix = 1;
	my %members;
	my $found = 0;
	my $callfile = "user-EXTEN-CIDNUM-DNIS-TIME-RAND";

	while (<INCL>)
	{
		if (/^format\s*=\s*(\S+)/)
		{
			$fmt = $1;
		}
		elsif (/^mix\s*=\s*(\d+)/)
		{
			$mix = $1;
		}
		elsif (/^callfile\s*=\s*(\S+)/)
		{
			$callfile = $1;
		}
		elsif (/^member\s*=>?\s*(\S+)/)
		{
			$members{$1} = 1;
			if ($1 eq $exten)
			{
				$found = $exten;
				last;
			}
			elsif ($1 eq $cidnum)
			{
				$found = $cidnum;
				last;
			}
		}
	}
	close(INCL);

	# we're not in the inclusion list
	if (!$found)
	{
		return;
	}

	my $name;

	# get the name
	if ($getName)
	{
		open(VM, "</etc/asterisk/voicemail.conf");
		while (<VM>)
		{
			if (substr($_, 0, length($found)) eq $found)
			{
				($name) = (/^\d+\s*=>\s*[^,]*,([^,]*),/);
				last;
			}
		}
		close(VM);

		# there should only be one dash
		$name =~ s/-/_/g;
		$name =~ s/ /-/;
		$name =~ s/ /_/g;

		# no last name (or no name at all)
		if ($name !~ /-/)
		{
			$name .= "-";
		}
	}

	# the extension is the destination number on outbound calls
	$number ||= $exten;

	# setup call file
	$callfile =~ s/EXTEN/$exten/g;
	$callfile =~ s/CIDNUM/$cidnum/g;
	$callfile =~ s/DNIS/$number/g;
	$callfile =~ s/TIME/time()/eg;
	$callfile =~ s/RAND/int(rand(1000))/eg;
	$callfile =~ s/NAME/$name/g;

	$callfile =~ s/ /_/g;

	$self->monitor($channel, $callfile, $fmt, $mix);
}


# process a queue call
sub process_queue
{
	my ($self, $data, $channel) = @_;

	my $callfile = $self->get_var("CALLFILENAME");

	my (undef, $exten, $cidnum, $number, $time, $rand) = split("-", $callfile);
	my $found = $exten;

	# we can make this faster by doing a stat comparison of time stamp if we want
	open(INCL, "<$inclusion") || error("Can't read inclusion file $inclusion: $!");

	my $fmt = "wav";
	my $mix = 1;
	my %members;
	my $found = 0;

	while (<INCL>)
	{
		if (/^format\s*=\s*(\S+)/)
		{
			$fmt = $1;
		}
		elsif (/^mix\s*=\s*(\d+)/)
		{
			$mix = $1;
		}
		elsif (/^queuefile\s*=\s*(\S+)/)
		{
			$callfile = $1;
		}
		elsif (/^member\s*=>?\s*(\S+)/)
		{
			$members{$1} = 1;
			if ($1 eq $exten)
			{
				$found = $exten;
				last;
			}
			elsif ($1 eq $cidnum)
			{
				$found = $cidnum;
				last;
			}
		}
	}
	close(INCL);


	# get the name
	my $name;
	if ($getName)
	{
		open(VM, "</etc/asterisk/voicemail.conf");
		while (<VM>)
		{
			if (substr($_, 0, length($found)) eq $found)
			{
				($name) = (/^\d+\s*=>\s*[^,]*,([^,]*),/);
				last;
			}
		}
		close(VM);

		# there should only be one dash
		$name =~ s/-/_/g;
		$name =~ s/ /-/;
		$name =~ s/ /_/g;

		# no last name (or no name at all)
		if ($name !~ /-/)
		{
			$name .= "-";
		}
	}

	# the extension is the destination number on outbound calls
	$number ||= $exten;

	# setup call file
	$callfile =~ s/EXTEN/$exten/g;
	$callfile =~ s/CIDNUM/$cidnum/g;
	$callfile =~ s/DNIS/$number/g;
	$callfile =~ s/TIME/time()/eg;
	$callfile =~ s/RAND/int(rand(1000))/eg;
	$callfile =~ s/NAME/$name/g;

	$callfile =~ s/ /_/g;

	$self->monitor($channel, $callfile, $fmt, $mix, 1);
}


# log into the manager
sub mgr_login
{
	my ($self, $die) = @_;

	my $sock = $self->{manager};
	close($sock) if $sock;

	$sock = IO::Socket::INET->new(
		PeerAddr => "localhost",
		PeerPort => 5038,
		Timeout => 5,
		Proto => "tcp",
	);

	# couldn't connect to asterisk manager
	unless ($sock)
	{
		if ($die)
		{
			die "Can't connect to asterisk manager: $!";
		}
		else
		{
			print STDERR "Can't connect to manager: $!\n";
		}
		sleep 10;

		return;
	}

	<$sock>;

	print $sock (<< "EOF");
Action: Login\r
Username: remote_mgr\r
Secret: 0chanc3yo\r
\r
EOF

	while (<$sock>)
	{
		last if /^\r?$/;
	}

	$self->{manager} = $sock;

	return 1;
}


sub monitor
{
	my ($self, $bridge, $fname, $fmt, $mix, $force_bridge) = @_;
open F,">>/tmp/deb";print F "monitor($bridge, $fname, $fmt, $mix)\n";close(F);

	if ($manager)
	{
		my $sock = $self->{manager};

		print $sock (<< "EOF");
Action: Monitor\r
File: $fname\r
Format: $fmt\r
Mix: $mix\r
Channel: $bridge\r
\r
EOF

		eval
		{
			$SIG{ALRM} = sub { die };
			alarm(3);

			while (<$sock>)
			{
				last if /^\r?$/;
			}
			alarm(0);
		};

		close($sock);
	}

	# if we need to specify the channel but aren't using the manager
	elsif (!$manager && $force_bridge)
	{
		system("perl", "/var/lib/asterisk/agi-bin/monitor_chan.agi", $bridge, $fname, $fmt, $mix, 1);
	}

	else
	{
		$mix = ($mix ? "m" : "");
		print "EXEC Monitor $fmt|$fname|$mix\n";
	}

	return 1;
}


sub get_var
{
	my ($self, $field) = @_;

	print "GET VARIABLE " . uc($field) . "\n";
	my $data = <STDIN>;
open F,">>/tmp/deb";print F "get_var($field) = $data";close(F);

	# got data back
	if (substr($data, 0, 14) eq "200 result=1 (")
	{
		return substr($data, 14, index($data, ")") - 14);
	}
	return undef;
}


=pod
agi_network: yes
agi_request: agi://localhost:6969
agi_channel: SIP/test-874d
agi_language: en
agi_type: SIP
agi_uniqueid: 1150693484.162
agi_callerid: Mike (home) Joyce <7106>
agi_dnid: 958584146807
agi_rdnis: unknown
agi_context: internal
agi_extension: 958584146807
agi_priority: 1
agi_enhanced: 0.0
agi_accountcode: 



record_all.conf.example:

; format for recording
format = wav

; mix files?
mix = 0

; members to record (this can include callerid values!)
member = 7777
member = 8584146800

