#!/usr/bin/perl

package server;
@ISA = qw(Net::Server::PreFork);

use strict;
use vars qw(@ISA);
use Asterisk::AGI;
use Net::Server::PreFork;

# directory to find .pagi's
my $PAGI_DIR = "/var/lib/asterisk/agi-bin/pagi";

# Have to get asterisk version here; if we try to get it inside a pagi it'll crash
my ($ASTERISK_VERSION) = `/usr/sbin/asterisk -V` =~ /.*?(\d+\.\d+).*/;
sub asterisk_version { $ASTERISK_VERSION }

# should we debug?
my $DEBUG = 1;

# reload pagis on hup
$SIG{'HUP'} = \&read_pagis;

# port to list on
my $PORT = 6969;

# global functions
my %functions;

# run the server
server->run();
exit;

# log things
sub debug
{
	if ($DEBUG)
	{
		open(LOG, ">>/tmp/pagi.err");
		print LOG localtime() . ": DEBUG: $_[0]\n";
		close(LOG);
	}
}

# re-read our pagis
sub read_pagis
{
	debug("Reading in PAGIs");
	%functions = ();

	opendir(PAGI, $PAGI_DIR) || die "Unable to open $PAGI_DIR: $!";
	foreach my $pl (grep { /\.pagi$/ } readdir(PAGI))
	{
		if ($pl =~ /^(.*)\.pagi$/)
		{
			$functions{$1} = do($PAGI_DIR . "/$pl");
		}
	}
	closedir(PAGI);
}

# set up some server parameters
sub configure_hook
{
	my $self = shift;

	$self->{server}->{port}		= ["127.0.0.1:$PORT"];		# port and addr to bind
	$self->{server}->{chdir}	= '/';			# chdir to root
	$self->{server}->{user}		= 'nobody';		# user to run as
	$self->{server}->{group}	= 'nobody';		# group to run as
	$self->{server}->{setsid}	= 1;			# daemonize
	
	$self->{AGI} = new Asterisk::AGI;
	
	open(STDIN,  '</dev/null') || die "Can't close STDIN [$!]";
	open(STDOUT, '>/dev/null') || die "Can't close STDOUT [$!]";
}

# do this when the connection is made
sub post_accept_hook
{
	my $self = shift;
	read_pagis();
}


# process the request
sub process_request
{
	my $self = shift;
	my $AGI = $$self{AGI};

	# read in agi params
	while (<STDIN>)
	{
		# standard agi format of data passed
		if (/^(\S+): (.*)$/)
		{
			$ENV{$1} = $2;
		}
		else
		{
			last;
		}
	}

	# evaluate our script!
	if ($ENV{'agi_network_script'})
	{
		$ENV{'agi_network_script'} =~ s|^/||;
		chomp($ENV{'agi_network_script'});
		$ENV{'agi_network_script'} =~ s/\r//g;

		my ($func, $params) = split(/\?/, $ENV{'agi_network_script'},2);
		$func =~ s/\.pagi$//;
		my %params = map { (split(/=/, $_, 2))[0, 1] } split(/&/, $params);

		# evaluate users function
		eval { &{$functions{$func}}($AGI, %params); };
	}
}


sub get_var
{
	my ($self, $field) = @_;

	print "GET VARIABLE " . uc($field) . "\n";
	my $data = <STDIN>;

	# got data back
	if (substr($data, 0, 14) eq "200 result=1 (")
	{
		return substr($data, 14, index($data, ")") - 14);
	}
	return undef;
}
