use XML::Simple;
use strict;
my %name_map = ();
my $license_perm_map = {};

############### GET XML FILE FUNCTIONS ###############
sub get_root
{
	my $server_id = shift;
	my $root;

	$root = "/data/$server_id" if($server_id =~ /^\d+$/);
	$root .= "/etc/asterisk";
	
	return $root;
}

sub get_prefix
{
	my $server_id = shift;
	return get_root($server_id) . "/hud3.0";
}

sub get_file
{
	my $file = shift;
	my $server_id = shift;
	my $AGI = shift || {};

	my $prefix = get_prefix($server_id);	
	
	my @stat = stat("$prefix/$file");
	if(!exists $AGI->{$file . "_timestamp"} || $AGI->{$file . "_timestamp"} != $stat[9])
	{
		# take in permissions xml
		my $xml = new XML::Simple;
		my $data = $xml->XMLin("$prefix/$file");
		$AGI->{$file . "_cache"} = $data;
		$AGI->{$file . "_timestamp"} = $stat[9];
	}

	return $AGI->{$file . "_cache"};
}

sub get_p3
{
	my $data = get_file("p3.xml", shift, shift);
	# this is required for uniformity between single and mutli-linked servers - Andre
	if(ref($data->{permissions}) eq "HASH") {
		$data->{permissions} = [$data->{permissions}];
	}

	return $data->{permissions};
}

sub get_u3
{
	my $data = get_file("u3.xml", shift, shift);
	$data->{users} = $data->{server};
	# this is required for uniformity between single and mutli-linked servers - Andre
	# a single server will have a structure as { id => server id, user => {...} }
	# while a mutl-linked server will have a sturture as { server id => { user => {...} }, server id2 => { user => {...} } }
	if(exists $data->{users}->{id}) {
		$data->{users} = { $data->{users}->{id} => $data->{users} };
	}

	# we also need uniformity, between a single and multi list of extensions - Andre
	# a single extension server will have a struct as { user => { firstName => ..., id => ...., extension => ... } }
	# whiel a multi extension server will have a struct as { user => { id1 => {...} }, id2 => {....} } } 
	foreach my $srv (keys %{$$data{users}}) {
		# we assume if 'id' exists, there must be only 1 extension
		# otherwise it would only by actual user ID number (/d+/)
		if(exists $$data{users}->{$srv}->{user}->{id}) {
			my $ndx = $$data{users}->{$srv}->{user}->{id};
			$$data{users}->{$srv}->{user} = { $ndx => $$data{users}->{$srv}->{user} };
		}
	}

	return $data->{users};
}

sub get_features
{
	# the XML format follows as { feature name => { 'permission' => perm1,perm2,...,permN } }
	# we need to make a data structure { perm1 => feature name1,feature name2,...,feature nameN }
	# the ol' switcheroo - Andre
	my $server_id = shift;
	my $AGI = shift || {};
	my $feature_data = get_file("f3.xml", $server_id, $AGI);

	#TODO: This is a hack. I know it (because why does this function know how we save data?), 
	# but let's not repopulate a hash if there is no change -Andre
	return 1 if($AGI->{"f3.xml_timestamp"} == $AGI->{"lic_perm_timestamp"});
	$AGI->{"lic_perm_timestamp"} = $AGI->{"f3.xml_timestamp"};
	
	$feature_data = $feature_data->{feature};
	foreach my $fc (keys %{$feature_data})
	{
		# we are expecting strings of type "perm1,perm2,...,permN"
		next if(ref($feature_data->{$fc}->{permission}) eq "HASH");
			
		my @perms = split(",",$feature_data->{$fc}->{permission});

		foreach (@perms) {
			$license_perm_map->{$_} .= "," if($license_perm_map->{$_});
			$license_perm_map->{$_} .= $fc;
		}
	}

	# this populates a global variable. There is no return.
	return 1;
}

############### END: GET XML FILE FUNCTIONS ###############




sub get_permissions
{
	my $server_id = shift;
	# if an AGI is passed, pagi is calling this. If not, then agi is -Andre
	my $AGI       = shift || {};

	# get data from u3.xml and p3.xml
	my $data = {};
	$data->{permissions} = get_p3($server_id, $AGI);
	# Felix Nilam - 11/16/2009
	# get the user license
	$data->{users} = get_u3($server_id, $AGI);
	
	# get the feature to permissions mapping
	# Felix Nilam - 12/09/2009
	# check if f3.xml file exists
	if(-e get_prefix($server_id) . "/f3.xml")
	{
		get_features($server_id, $AGI);
	}
	return $data;
}





sub get_ast_hdrs
{
	# a pagi script might call this. All asterisk headers have already been entered into the EVN variable.
	# this way, this will work with either an agi or pagi call - Andre
	my $env = shift;
	my %info;

	unless(defined $env) {
		# get asterisk headers
		while (<STDIN>)
		{
			$info{$1} = $2 if /^(.*): (.*)$/;
			dbg("$1 = $2");
	
			last if /^\r?\n/;
		}
	} else {
		%info = %$env;
	}

	$info{agi_channel} =~ s/-.*$//;
	
	return %info;
}


sub ssend
{
	my $data = shift;
	print STDOUT "$data\n";
	my $line = <STDIN>;

	return $line;
}

# debug command
sub dbg
{
	my $log = shift;
	chomp($log);
	open F, ">>/tmp/agi.log";
	print F localtime() . ": $log\n";
	close(F);
}

# returns the CallerID (extension number) of the person paging
sub get_cid
{
	my $info = shift;

	my $cid = $info->{agi_callerid};
	$cid =~ s/^.*<(\d+)>$/$1/;

	if (!$cid)
	{
		dbg("no valid callerid");
	}

	return $cid;
}




# verify user has proper rights to perform function
sub check_rights
{
	my ($ref, $ext, $exts, $right_name, $must_be_group, $must_be_individual) = @_;

	# Felix Nilam - 12/09/2009
	# check if $ext is allowed to $right_name based on the user license
	# if not, return 0;
	# are we looking at a permission that is feature-restricted?
	if($license_perm_map->{$right_name})
	{
		# get all exts with feature types that allow this permission $right_name.
		# we need to loop through mutliple servers, possibly - Andre
		my %user_allowed;
		foreach my $srv (keys %{$ref->{users}}) {
			my $inner_ref = $ref->{users}->{$srv};
			%user_allowed = (%user_allowed, get_exts_from_user($inner_ref, $license_perm_map->{$right_name}));
		}
		
		# if %user_allowed has extensions in it (meaning users follow license rules) 
		# [some servers have only non-license users] and
		# the extension is not found, $ext cannot perform $right_name
		if(keys(%user_allowed) && !$user_allowed{$ext})
		{
			return 0;
		}
	}
	
	# from this point we make the list/hash %allowed and fill it with all the extensions that 
	# $ext CAN perform $right_name on. Then we compare this list/hash with the list/hash $exts
	# the $ext is TRYING to perform the right on. If all exts in $exts do not also exist in %allowed
	# this will return 0 - Andre
	
	set_name_map($ref);
	
	# find out who's allowed
	my %allowed;
	my @groups;
	
	# we need to loop through multiple servers, possibly - Andre
	foreach my $srv (@{$ref->{permissions}}) {
		# first find the groups that $ext has $right_name over
		# this means: search through every group that the $ext belongs to (ie, the parent groups)
		# then adding to the list every group that the parent groups have that $right_name over (ie, the child groups)
		# the $ext may belong to many groups over many servers - Andre
		@groups = (@groups, @{get_childs_from_parent($srv, $ext, $right_name)});
	}

	# now that we have a list of child groups, get their members
	# unfortunately groups are ordered by name, there can be many groups with the same name on different servers
	foreach my $id (@groups) {
		%allowed = (%allowed, map { $_ => 1 } get_exts_from_group($ref, $id));
	}

	# make sure we have permission
	foreach (keys %{$exts})
	{
		
		unless (
			($must_be_group      && $exts->{$_} == 1 && $allowed{$_}) ||
			($must_be_individual && $exts->{$_} == 0 && $allowed{$_}) ||
			(!$must_be_group && !$must_be_individual && $allowed{$_})
		)
		{
			# not allowed
			return 0;
		}
	}
	
	return 1;
}

sub get_exts_from_group
{
	my $ref = shift;
	my $id  = shift;

	# we need to find that child group. loop through the servers and compare group ids
	foreach my $srv (@{$ref->{permissions}}) {
		my $pos = $srv->{group};
			
		# is there only 1 group on this server?
		my $one = exists $pos->{name};
		my $group_id;
			
		# loop through the groups
		foreach my $found (keys %{$pos}) {
			if($one) {
				$group_id = $pos->{id};
			} else {
				$group_id = $pos->{$found}->{id};
			}
			last if($id == $group_id);
		}
		
		# is the child group on this server?
		next unless($id == $group_id);
		# get the members of this group, found on this server.
		return get_ref($srv, "group" . (!$one ? "->$name_map{$id}" : "") . "->members->ext");
	}
}

sub set_name_map
{
	my $ref = shift;
	
	# map group ids to names, unfortunately there is ID uncomformity in the p3.xml
	# groups are labeled by their names, but a group's group is labeled by its id - Andre
	foreach my $r (@{$ref->{permissions}}) {
		my $pos = $r->{group};
		my $one = exists $pos->{name};	
		foreach my $key (keys %{$pos}) {
			if($one) {
				$name_map{$pos->{id}} = $pos->{name};
				last;
			} else {
				$name_map{$pos->{$key}->{id}} = $key;
			}
		}
	}
}

# get child groups of parent group. Searches for parent groups that a passed-in extension belongs to
sub get_childs_from_parent
{
	my ($ref, $ext, $right_name) = @_;

	# which groups have this right?
	my (@groups, $loc, $type, $one, $members);
	my $pos = $ref->{group};
	
	# is there only 1 group on this server?
	$one = exists $pos->{name};
	
	# search through parent groups
	foreach my $key (keys %{$pos}) {
		# if there is a key 'name', it implies only 1 group`
		# this will break if someone has only 1 group called 'name'. Likelyhood is low,
		# since a user cannot delete group 'Basic Human Rights' - Andre
		if($one) {
			$loc = $pos->{permissions};
			$members = $pos->{members}->{ext};
		} else {
			$loc = $pos->{$key}->{permissions};
			$members = $pos->{$key}->{members}->{ext};
		}
		
		my $ext_in_group = 0;
		# is the ext that is committing this act of pemission exist in this group?
		foreach my $mem (split(",",$members)) {
			if($mem == $ext) {
				$ext_in_group = 1;
				last;
			}
		}
		
		next unless($ext_in_group);
		
		# if yes, get the ids of all child groups that allow this permission to be acted upon it
		my @permissions = ref($loc) eq "ARRAY" ? @$loc : ($loc);
		# search through each group's groups
		foreach (@permissions) {
			# search through the permissions associated
			foreach my $perm (split(",",$_->{ids})) {
				# is it there? # add the id of the group
				if($perm eq $right_name) {
					push(@groups, $_->{group});
					last;
				}
			}
		}
		
		last if $one;
	}	

	# return the list of child groups found on this server - Andre
	return \@groups;
}

# get extensions from users with allowed feature components
sub get_exts_from_user
{
	my ($ref, $feature_name) = @_;

	my (%ret, $one, $ext_loc, $lic_loc);
	my $pos = $ref->{user};
	
	# is there only 1 user on this server?
	# 1 user structure is { 'id' => user id, 'firstName' => first name }
	# many user structure is { user id => { 'firstName' => first name, ... }, user id 2 => {...} }
	my $one = exists $pos->{id};
	
	# search through users
	foreach my $user (keys %{$pos})
	{
		if($one) {
			$lic_loc = $pos->{license};
			$ext_loc = $pos->{extension};
		} else {
			$lic_loc = $pos->{$user}->{license};
			$ext_loc = $pos->{$user}->{extension};
		}
		
		my @exts = ref($ext_loc) eq "ARRAY" ? @$ext_loc : ($ext_loc);
		# search through each feature type of this user
		# if we did not find any feature type at the lic_loc, then we may be dealing
		# with a server with only non-license users - Andre
		foreach my $feature (split(",",$lic_loc))
		{
			# if features are found, then we are dealing with a server that 
			# must go through feature-restriction.
			
			# compare this user's feature types to the passed-in $feature_name
			# does this user have this feature type?
			# if so, add all its extension numbers as { ext1 => 1, ext2 =>1,... }
			# else, add all its extension numbers as { ext1 => 0, ext2 => 0,... }
			my $found = 0;
			$found = 1 if(grep { $_ eq $feature } split(",",$feature_name));
			
			foreach (@exts) {
				$ret{$_->{number}} = $found;
			}

			last if $found;				
		}

		# we loop only once if 1 user, because this will loop through the keys 'firstName', 'lastName', ...
		# as opposed to looping appropiately through user id 1, user id 2,...
		last if $one;
	}

	return %ret;
}

# sweet function to return a list of shiz from refs
sub get_ref
{
	my ($ref, @list) = @_;

	@list = map { split("->") } @list;
	
	my $var = $ref;
	foreach (@list)
	{
		if (ref($var) eq "HASH")
		{
			$var = $var->{$_};
		}
		else
		{
			$var = undef;
			last;
		}
	}

	my @list = ref($var) eq "ARRAY" ? @{$var} : ($var);
	foreach (@list)
	{
		s/R\((\d+),(\d+)\)/join(",", $1 .. $2)/eg;
	}
	return map { split(",") } @list;
}

# produce a list of extensions from extensions/extension groups
# returns a hash of extensions => 1 if in a group, => 0 if not, group extensions are removed
sub get_exts
{
	my ($ref, @exts) = @_;

	my %out;
	foreach my $srv (@{$ref->{permissions}}) {
		my %set = %{_get_exts($srv, @exts)};
		# _get_exts will remove a group extension from the structure it returns, but
		# that same extension is never removed from @exts, so it will possibly appear
		# as $out{group extension} = 0, if iterating over mutliple servers, which is bad.
		# group extensions need to be removed.
		# If the ext does not exist in the hash, remove it from @exts - Andre
		for(my $i = 0; $i < @exts; $i++) {
			unless(exists $set{$exts[$i]}) {
				delete $out{$exts[$i]};
				splice(@exts, $i, 1);
				$i--;
			}
		}
		%out = (%out, %set);
	}

	return \%out;
}

sub _get_exts
{
	my ($ref, @exts) = @_;
	$ref ||= {};
	
	my $pos = $ref->{group} || {};
	my (%struct, $loc);
	
	# assume none of the exts are group extensions
	foreach my $ext (@exts) {
		$struct{$ext} = 0;
	}
	
	my $one = exists $pos->{name};	
	foreach my $key (keys %{$pos}) {
		# if there is a key 'name', it implies only 1 group`
		# this will break if someone has only 1 group called 'name'. Likelyhood is low,
		# since a user cannot delete group 'Basic Human Rights' - Andre
		my $inner_pos;
		if($one) {
			# is this a requested group extension?
			next unless(exists $struct{$pos->{ext}});
			delete $struct{$pos->{ext}};
			$inner_pos = $pos->{members};
		} else {
			# is this a requested group extension?
			next unless(exists $struct{$pos->{$key}->{ext}});
			delete $struct{$pos->{$key}->{ext}};
			$inner_pos = $pos->{$key}->{members};
		}
		
		if(ref($inner_pos) eq "ARRAY") {
			foreach my $set (@{$inner_pos}) {
				foreach (get_ref($set, "ext")) {
					$struct{$_} = 1;
				}
			}
		} else {
			foreach (get_ref($inner_pos, "ext")) {
				# add all extensions assoicated with this group
				$struct{$_} = 1;
			}
		}

		last if $one;
	}

	return \%struct;
}


# given an extension, determine the primary ext. for the user
# if primary can't be found, just return the given $exten
#
# if called in a list context, returns ($primary_ext, $mobile_number)
sub get_primary_ext
{
	my $exten = shift;
	my $server_id = shift;
	my $AGI = shift;

	my $u3 = get_u3($server_id, $AGI);

	# foreach server
	foreach my $srv (keys %{$u3})
	{
		# foreach usser on that server
		foreach my $uid (keys %{$u3->{$srv}->{user}})
		{
			my $user = $u3->{$srv}->{user}->{$uid};
			my $list = $user->{extension};
			# put the extensions in an array ref.
			# xml will set this {extension} as a hash, if there is only 1 extension
			$list = [$list] unless(ref($list) eq "ARRAY"); 
			
			# foreach extension owned by that user
			foreach my $ext (@$list)
			{
				if ($ext->{number} eq $exten)
				{
					if ($user->{'primary-ext'} =~ /^\d+$/)
					{
						return (wantarray and 
							exists $user->{mobile} and 
							exists $user->{mobile}->{phone}) ?
						  ($user->{'primary-ext'},$user->{mobile}->{phone}) :
						  $user->{'primary-ext'};
					}
					else
					{
						return $exten;
					}
				}
			}
		}
	}

	# I dunno.	
	return $exten;
}

1;
