#!/usr/bin/perl
#
# Samy Kamkar's XMLin() replacement
#
# This uses XML::Parser::Expat with my own functions to generate
# an identical list to XML::Parser's XMLin() function. 
#

use XML::Parser::Expat; 

# here is the drop-in replacement function for XMLin()
sub SamyXMLin
{
	unless (open(FOO, $_[0]))
	{
		$@ = "Couldn't read $_[0]: $!";
		return undef;
	}

	my $parser = new XML::Parser::Expat;
	$parser->setHandlers(
		'Start' => \&sh,
		'End' => \&eh,
		'Char' => \&ch,
	);
	$parser->{xml} = { };
	$parser->{xmllast} = [ $parser->{xml} ];
#	$parser->parse(*FOO);
	$parser->parse(join "", <FOO>);
	close(FOO);

	return $parser->{xml}->{opt};
}


# this is used to read in beginning xml tags
sub sh
{
	my ($p, $el, %attr) = @_;

	push @{$p->{xmllast}}, $p->{xml};
	if (keys %attr == 0)
	{
		$p->{set} = 1;
		$p->{xml}->{$el} ||= {};
		$p->{xml} = $p->{xml}->{$el};
	}
	elsif (keys %attr == 1 && $attr{name})
	{
		$p->{xml}->{$el} ||= { $attr{name} };
		$p->{xml}->{$el}->{$attr{name}} ||= {};
		$p->{xml} = $p->{xml}->{$el}->{$attr{name}};
	}
	else
	{
		if ($attr{name})
		{
			$p->{xml}->{$el}->{delete $attr{name}} = \%attr;
		}
		else
		{
			$p->{xml}->{$el} = \%attr;
		}
		$p->{xml} = $p->{xml}->{$el};
	}
}


# go down a level of refs (ending xml tag)
sub eh
{
	if ($_[0]->{set} && ref($_[0]->{xml}) eq "ARRAY")
	{
		$_[0]->{set} = 0;
		push @{$_[0]->{xml}}, {};
	}

	$_[0]->{current} = $_[0]->{xml};
	$_[0]->{xml} = pop(@{$_[0]->{xmllast}});
}


# this is data within a starting and ending xml tag, e.g. <blah>THIS</blah>
sub ch
{

	my $char = substr($_[1], 0, 1);
	if ($char ne "\t" && $char ne "\n" && $char ne " ")
	{
		$_[0]->{set} = 0;

		my $loc = $_[0]->{Context}->[-1];
		my $ref = ref($_[0]->{xmllast}->[-1]->{$loc});
		if ($ref eq "ARRAY")
		{
#			$_[0]->{xmllast}->[-1]->{$loc}->[-1] = $_[1];
			push @{$_[0]->{xmllast}->[-1]->{$loc}}, $_[1];
		}
		elsif ($ref eq "HASH")
		{
			if ($_[0]->{current} eq $_[0]->{xml})
			{
				$_[0]->{xmllast}->[-1]->{$loc} =
					[ $_[0]->{xmllast}->[-1]->{$loc}, $_[1] ];
			}
			else
			{
				$_[0]->{xmllast}->[-1]->{$loc} = $_[1];
			}
		}
		else
		{
			$_[0]->{xmllast}->[-1]->{$loc} =
				[ $_[0]->{xmllast}->[-1]->{$loc}, $_[1] ];
		}
	}
}

1;
